#!/bin/sh
JTI_NB_CHAR=16

show_help() {
    echo "Usage: ${0} [-h] -i <issuer> -k <key_id> [-l <lrr_uuid>] [-p <private_key>]"
    echo "    -h: help"
    echo "    -i: issuer, final issuer will be 'prestager:<issuer>'"
    echo "    -k: key id"
    echo "    -l: LRR UUID, if missing will try to build it from host platform"
    echo "    -d: delete private key"
    echo "    -p: private key (default: ./<key_id>.pem)"
    echo "    -o: output file (default: ./token-<lrr_uuid>.jwt)"
    exit 0
}

ICS_DELETE_PRIVATE_KEY=0

while getopts 'i:k:l:p:o:dh' opt; do
  case $opt in
    i)
        ICS_ISSUER="prestager:${OPTARG}"
        ;;
    k)
        ICS_KID=${OPTARG}
        ;;
    l)
        ICS_UID=${OPTARG}
        ;;
    p)
        ICS_PRIVATE_KEY=${OPTARG}
        ;;
    o)
        ICS_TOKEN=${OPTARG}
        ;;
    h)
        show_help
        ;;
    d)
        ICS_DELETE_PRIVATE_KEY=1
        ;;
    *)
        show_help
        ;;
  esac
done

#########################
# GATEWAY SPECIFIC CODE #
#########################
ACT_BST_JWT_FACTORY_TOKEN_FILENAME="act_bst_token.jwt"

ACT_BST_SAVE_PATH="/opt/bscs-storage"

build_uuid()
{
    ICS_UID="24e124-$(ifconfig eth0 | grep HWaddr | awk '{ print $5 }' | sed 's/://g' | tr '[A-Z]' '[a-z]')"
    return $?
}

store_jwt_token()
{
    file=$1
    mkdir -p "${ACT_BST_SAVE_PATH}"
    cp  "$file" "${ACT_BST_SAVE_PATH}/${ACT_BST_JWT_FACTORY_TOKEN_FILENAME}"
    return 0
}
################################
# end of GATEWAY SPECIFIC CODE #
################################

if [ -z "$ICS_ISSUER" ]; then
    echo "Issuer is mandatory."
    show_help
fi

if [ -z "$ICS_KID" ]; then
    echo "Key ID is mandatory."
    show_help
fi

ACT_BST_EMBEDDED_MODE=0
if [ -z "$ICS_UID" ]; then
    ACT_BST_EMBEDDED_MODE=1
    if ! build_uuid; then
        echo "Can not build LRR UUID from host, exiting..."
        exit 1
    fi
fi

if [ -z "$ICS_PRIVATE_KEY" ]; then
    ICS_PRIVATE_KEY="./${ICS_ISSUER}-$ICS_KID.pem"
fi

if [ ! -f "$ICS_PRIVATE_KEY" ]; then
    echo "Can not find private key ${ICS_PRIVATE_KEY}."
    exit 1
fi

if [ -z "$ICS_TOKEN" ]; then
    ICS_TOKEN="./token-${ICS_UID}.jwt"
fi

build_header()
{
    echo "{\"typ\":\"JWT\",\"alg\":\"RS256\",\"kid\":\"${ICS_KID}\"}"
}

build_payload()
{
    echo "{\"iss\":\"${ICS_ISSUER}\",\"sub\":\"${ICS_UID}\",\"jti\":\"$(head /dev/urandom | tr -dc A-Za-z0-9 | head -c ${JTI_NB_CHAR} ; echo '')\",\"iat\":${ICS_IAT}}"
}

hmac_rsa256_sign()
{
    input=$1
    printf '%s' "${input}" | openssl dgst -sign "${ICS_PRIVATE_KEY}" -keyform PEM -sha256 -binary
}

ICS_IAT=$(date +%s)

header=$(build_header)
payload=$(build_payload)

header_base64=$(echo "${header}" | tr -d '\n' | tr -d '\r' | openssl base64 | tr +/ -_ | tr -d '=' | tr -d '\n')
payload_base64=$(echo "${payload}" | tr -d '\n' | tr -d '\r' | openssl base64 | tr +/ -_ | tr -d '=' | tr -d '\n')

header_payload="${header_base64}.${payload_base64}"
signature=$(hmac_rsa256_sign "${header_payload}" | openssl base64 | tr -d '=' | tr '/+' '_-' | tr -d '\n')

echo "${header_payload}.${signature}" > "$ICS_TOKEN"

if [ $ACT_BST_EMBEDDED_MODE -eq 1 ]; then
    if store_jwt_token $ICS_TOKEN; then
        echo "JWT Token successfully generated and saved to persistent storage."
        if [ $ICS_DELETE_PRIVATE_KEY -eq 1 ]; then
            echo "Deleting local private key"
            rm ${ICS_PRIVATE_KEY}
        fi
    fi
fi
